Package: gWidgetsRGtk2
Version: 0.0-86.1
Title: Toolkit Implementation of gWidgets for RGtk2
Author: Michael Lawrence, John Verzani
Maintainer: ORPHANED
Depends: methods, grDevices, utils, graphics, RGtk2, gWidgets,
        cairoDevice
Description: Port of the gWidgets API to the RGtk2 toolkit.
License: GPL (>= 2)
LazyLoad: yes
Packaged: 2020-05-20 08:34:37 UTC; ripley
NeedsCompilation: no
Repository: CRAN
Date/Publication: 2020-05-20 10:34:52
RoxygenNote: 6.0.1
